@@ -305,7 +305,7 @@ void SeedAlignmentPolicy::parseString(
 	}
 	
 	multiseedMms      = DEFAULT_SEEDMMS;
-	multiseedLen      = DEFAULT_SEEDLEN;
+	multiseedLen      = gDefaultSeedLen;
 	
 	EList<string> toks(MISC_CAT);
 	string tok;
@@ -409,6 +409,25 @@ void SeedAlignmentPolicy::parseString(
 				}
 				// Set type to =quality
 				penMmcType = COST_MODEL_QUAL;
+				/* The match penalty may get set before
+				 * the --ignore-quals parameter gets parsed.
+				 * Since the match penalty policy depends on
+				 * the whether or not --ignore-quals was set
+				 * we do a check here and adjust the parameter
+				 * accordingly.
+				 */
+				if (ignoreQuals) {
+					if (gVerbose)
+						cerr << "Changing MMP=Q," << penMmcMax << " to ";
+					penMmcMin = penMmcMax;
+					penMmcType = COST_MODEL_CONSTANT;
+					if (gVerbose) {
+						cerr << "MMP=C," << penMmcMax
+						     << " because of --ignore-quals"
+						     << endl;
+					}
+
+				}
 			} else if(ctoks[0][0] == 'R') {
 				// Set type to=Maq-quality
 				penMmcType = COST_MODEL_ROUNDED_QUAL;
@@ -534,9 +553,9 @@ void SeedAlignmentPolicy::parseString(
 		 *          interval is determined by IVAL.
 		 */
 		else if(tag == "SEED") {
-			if(ctoks.size() > 2) {
+			if(ctoks.size() > 1) {
 				cerr << "Error parsing alignment policy setting "
-				     << "'" << tag.c_str() << "'; RHS must have 1 or 2 tokens, "
+				     << "'" << tag.c_str() << "'; RHS must have 1 token, "
 					 << "had " << ctoks.size() << ".  "
 					 << "Policy: '" << s.c_str() << "'" << endl;
 				assert(false); throw 1;
@@ -553,12 +572,6 @@ void SeedAlignmentPolicy::parseString(
 					throw 1;
 				}
 			}
-			if(ctoks.size() >= 2) {
-				istringstream tmpss(ctoks[1]);
-				tmpss >> multiseedLen;
-			} else {
-				multiseedLen = DEFAULT_SEEDLEN;
-			}
 		}
 		else if(tag == "SEEDLEN") {
 			if(ctoks.size() > 1) {
@@ -701,7 +714,6 @@ int main() {
 		assert_eq(DEFAULT_REF_GAP_CONST,      penRfExConst);
 		assert_eq(DEFAULT_REF_GAP_LINEAR,     penRfExLinear);
 		assert_eq(DEFAULT_SEEDMMS,            multiseedMms);
-		assert_eq(DEFAULT_SEEDLEN,            multiseedLen);
 		assert_eq(DEFAULT_IVAL,               msIval.getType());
 		assert_eq(DEFAULT_IVAL_A,             msIval.getCoeff());
 		assert_eq(DEFAULT_IVAL_B,             msIval.getConst());
@@ -755,7 +767,6 @@ int main() {
 		assert_eq(DEFAULT_REF_GAP_CONST_BADHPOLY,   penRfExConst);
 		assert_eq(DEFAULT_REF_GAP_LINEAR_BADHPOLY,  penRfExLinear);
 		assert_eq(DEFAULT_SEEDMMS,            multiseedMms);
-		assert_eq(DEFAULT_SEEDLEN,            multiseedLen);
 		assert_eq(DEFAULT_IVAL,               msIval.getType());
 		assert_eq(DEFAULT_IVAL_A,             msIval.getCoeff());
 		assert_eq(DEFAULT_IVAL_B,             msIval.getConst());
@@ -810,7 +821,6 @@ int main() {
 		assert_eq(DEFAULT_REF_GAP_CONST,      penRfExConst);
 		assert_eq(DEFAULT_REF_GAP_LINEAR,     penRfExLinear);
 		assert_eq(DEFAULT_SEEDMMS,            multiseedMms);
-		assert_eq(DEFAULT_SEEDLEN,            multiseedLen);
 		assert_eq(DEFAULT_IVAL,               msIval.getType());
 		assert_eq(DEFAULT_IVAL_A,             msIval.getCoeff());
 		assert_eq(DEFAULT_IVAL_B,             msIval.getConst());
@@ -864,7 +874,6 @@ int main() {
 		assert_eq(24.0f,                      penRfExConst);
 		assert_eq(12.0f,                      penRfExLinear);
 		assert_eq(DEFAULT_SEEDMMS,            multiseedMms);
-		assert_eq(DEFAULT_SEEDLEN,            multiseedLen);
 		assert_eq(DEFAULT_IVAL,               msIval.getType());
 		assert_eq(DEFAULT_IVAL_A,             msIval.getCoeff());
 		assert_eq(DEFAULT_IVAL_B,             msIval.getConst());
