@@ -834,7 +834,8 @@ void AlnSinkWrap::finishRead(
 			} else {
 				met.nconcord_uni++;
 				assert(!rs1_.empty());
-				if(rs1_.size() == 1) {
+
+				if(!concordSumm.bestUnchosenCScore().valid()) {
 					met.nconcord_uni1++;
 				} else {
 					met.nconcord_uni2++;
