@@ -1,10 +1,29 @@
+/*
+ * Copyright 2011, Ben Langmead <langmea@cs.jhu.edu>
+ *
+ * This file is part of Bowtie 2.
+ *
+ * Bowtie 2 is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * Bowtie 2 is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Bowtie 2.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
 #ifndef ASSERT_HELPERS_H_
 #define ASSERT_HELPERS_H_
 
-#include <iostream>
 #include <stdexcept>
 #include <string>
 #include <cassert>
+#include <iostream>
 
 /**
  * Assertion for release-enabled assertions
@@ -19,9 +38,9 @@ public:
  * all assertion error messages more helpful.
  */
 #ifndef NDEBUG
-#define ASSERT_ONLY(x...) x
+#define ASSERT_ONLY(...) __VA_ARGS__
 #else
-#define ASSERT_ONLY(x...)
+#define ASSERT_ONLY(...)
 #endif
 
 #define rt_assert(b)  \
