@@ -19,11 +19,6 @@
 
 #ifndef BT2_SEARCH_H_
 #define BT2_SEARCH_H_
-#ifdef WITH_TBB
-
-#include <tbb/tbb.h>
-#include <tbb/task_group.h>
-
 class multiseedSearchWorker {
 	int tid;
 
@@ -43,5 +38,4 @@ public:
 
 };
 
-#endif /* WITH_TBB */
 #endif /* BT2_SEARCH_H_ */
